@@ -6,6 +6,7 @@ from threading import Thread
 
 from PyQt4.QtCore import QObject, QTimer
 from data import DataParser, Collection, Resource
+from dbusmanager import DBusMonitor
 
 """ ConnectionManager gets request from ui layer. Does there request to network
     with python threads so it wont block the main ui thread. Returs data either to
@@ -13,12 +14,12 @@ from data import DataParser, Collection, Resource
 
 class ConnectionManager:
 
-    def __init__(self, controller, ui_handler):
+    def __init__(self, controller, ui_handler, logger, maemo_env):
         self.controller = controller
-        self.data_parser = DataParser(ui_handler)
+        self.data_parser = DataParser(ui_handler, logger)
         self.ui_handler = ui_handler
         self.tree_controller = ui_handler.tree_controller
-        self.log = controller.log
+        self.logger = logger
         self.client = None
 
         # Init poll timer
@@ -27,34 +28,81 @@ class ConnectionManager:
 
         # Init running threads to empty
         self.running_threads = []
+        self.queued_threads = []
+        
+        # DBus manager
+        self.dbus_monitor = DBusMonitor(self, logger, maemo_env)
+        
+        # Start the thread poller
+        self.poll_timer.start(100)
 
     def thread_poller(self):
+        self.dbus_monitor.iteration()
         for thread in self.running_threads:
             thread.join(0.01)
             if not thread.isAlive():
                 if thread.error != None:
-                    self.ui_handler.show_banner("Networking error, are you connected to the internet?")
+                    # If tree exists, lets store the request and do it again
+                    # when a connection is available
+                    if self.tree_controller.root_folder != None:
+                        self.queued_threads.append(thread)
+                    # Request a connection only once
+                    if len(self.queued_threads) == 1:
+                        self.request_connection()
                 if thread.response != None:
                     if thread.callback_parameters != None:
                         thread.callback(thread.response, thread.callback_parameters)
                     else:
                         thread.callback(thread.response)
                 self.running_threads.remove(thread)
-
+    
+    def request_connection(self):
+        self.dbus_monitor.request_connection()
+        
+    def connection_available(self):
+        return self.dbus_monitor.device_has_networking
+        
+    def set_connected(self, connected):
+        if connected:
+            self.ui_handler.hide_loading_ui()
+            if self.controller.login_done == False:
+                self.logger.network("Network connection established, starting authentication")
+                self.controller.start_trusted_auth()
+                self.ui_handler.try_trusted_login()
+            elif self.controller.connected == True and self.client != None:
+                if len(self.queued_threads) > 0:
+                    self.ui_handler.show_information_ui("Connection established, fetching queued tasks", True)
+                    self.logger.network("Network connection established, starting queued networking")
+                    for queued_thread in self.queued_threads:
+                        worker = NetworkWorker()
+                        worker.clone(queued_thread)
+                        worker.start()
+                        self.running_threads.append(worker)
+                    self.queued_threads = []
+                else:
+                    self.ui_handler.show_information_ui("Connection established, fetching content", True)
+                    self.logger.network("Network connection established, fetching root metadata")
+                    self.get_account_data()
+                    self.get_metadata("/", "dropbox")
+        else:
+            self.ui_handler.show_loading_ui("Waiting for a connection...", False)
+    
     def set_client(self, client):
         # Set client for usage, we are not connected
         self.client = client
         # Get account information
-        if self.get_accoutn_data():
-            # Start network thread poller
-            self.poll_timer.start(100)
-            # Start by fetching sandbox root contents
-            self.get_metadata("/", "dropbox")
+        if self.connection_available():
+            if self.get_account_data():
+                # Start by fetching sandbox root contents
+                self.get_metadata("/", "dropbox")
+            else:
+                self.ui_handler.show_loading_ui("Waiting for a connection...", False)
+                self.request_connection()
         else:
-            self.client = None
-            self.ui_handler.show_banner("Networking error, are you connected to the internet?")
-            
-    def get_accoutn_data(self):
+            self.ui_handler.show_loading_ui("Waiting for a connection...", False)
+            self.request_connection()
+
+    def get_account_data(self):
         try:
             self.data_parser.parse_account_info(self.client.account_info())
             return True
@@ -63,7 +111,7 @@ class ConnectionManager:
     
     def check_client(self):
         if not self.client:
-            self.log("ERROR - Tried networking without a ready connection")
+            self.logger.network_error("Tried networking without a ready connection")
             return False
         else:
             return True
@@ -111,8 +159,8 @@ class ConnectionManager:
                 if resp.status == 200: # ok
                     tree_item = self.data_parser.parse_metadata(resp.data, self.updated_open_folders)
                 else:
-                    self.log("NETWORK ERROR "+str(resp.status)+" - Could not fetch metadata")
-                    self.log(">> ", resp.body)
+                    self.logger.network_error(str(resp.status) + " - Could not fetch metadata")
+                    self.logger.network_error(">> ", resp.body)
                     self.show_information("Metadata fetch failed, network error", False)
         else:
             self.show_information("Metadata fetch failed, internal error", False)
@@ -133,7 +181,7 @@ class ConnectionManager:
                 if child_folder.tree_item.isExpanded():
                     self.updated_open_folders[child_folder.get_name()] = child_folder.path
         else:
-            self.log("ERROR - Could not find tree item for path", path)
+            self.logger.error("Could not find tree item for path", path)
 
     ### GET THUMBNAIL HANDLERS
     def get_thumbnail(self, path, root, size):
@@ -160,8 +208,8 @@ class ConnectionManager:
             if resp.status == 200:
                 self.data_parser.parse_thumbnail(resp, image_path)
             else:
-                self.log("NETWORK ERROR "+str(resp.status)+" - Could not fetch thumbnail for", image_path)
-                self.log(">> Reason:", resp.read())
+                self.logger.network_error(str(resp.status) + " - Could not fetch thumbnail for", image_path)
+                self.logger.network_error(">> Reason:", resp.read())
                 self.show_information("Thumbnail fetch failed, network error", False)
         else:
             self.hide_loading_ui()
@@ -195,8 +243,8 @@ class ConnectionManager:
                 except IOError:
                     self.show_information("Could not open " + store_path + " for store operation", False)
             else:
-                self.log("NETWORK ERROR "+str(resp.status)+" - Could not download file to", store_path)
-                self.log(">> Reason:", resp.read())
+                self.logger.network_error(str(resp.status) + " - Could not download file to", store_path)
+                self.logger.network_error(">> Reason:", resp.read())
                 self.show_information("Download failed, network error", False)
         else:
             self.hide_loading_ui()
@@ -210,7 +258,7 @@ class ConnectionManager:
         elif root == "dropbox":
             root_name = "DropBox"
         else:
-            self.log("ERROR - Upload file path/root parse error! Cannot continue.")
+            self.logger.error("Upload file path/root parse error! Cannot continue.")
             return
         file_name = local_file_path.split("/")[-1]
         self.show_loading_ui("Uploading file " + file_name + "\nto " + root_name + path)
@@ -220,7 +268,7 @@ class ConnectionManager:
             file_obj = open(local_file_path, "rb")
         except IOError:
             self.hide_loading_ui()
-            self.log("ERROR - Could not open " + local_file_path + " file for uploading. Aborting")
+            self.logger.error("Could not open " + local_file_path + " file. Aborting upload.")
             self.show_information("Could not open " + local_file_path + " file for uploading", False)
             return
         
@@ -243,8 +291,8 @@ class ConnectionManager:
                 self.get_automated_metadata(store_path, root)
                 self.show_information("Upload of " + file_name + " completed", True)
             else:
-                self.log("NETWORK ERROR "+str(resp.status)+" - Upload of " + params[0] + " to " + store_path + "failed.")
-                self.log(">> Reason:", resp.body)
+                self.logger.network_error(str(resp.status) + " - Upload of " + params[0] + " to " + store_path + "failed.")
+                self.logger.network_error(">> Reason:", resp.body)
                 self.show_information("Upload of " + file_name + " failed, network error", False)
         else:
             self.hide_loading_ui()
@@ -279,8 +327,8 @@ class ConnectionManager:
                 data.tree_item.setText(0, new_name)
                 self.show_information("Renamed " + old_name + " succesfully", True)
             else:
-                self.log("NETWORK ERROR "+str(resp.status)+" - Renaming failed.")
-                self.log(">> Reason:", resp.body)
+                self.logger.network_error(str(resp.status) + " - Renaming failed.")
+                self.logger.network_error(">> Reason:", resp.body)
                 self.show_information("Renaming of " + old_name + " failed, network error", False)
             # Stop rename loading anim
             data.set_loading(False)
@@ -315,8 +363,8 @@ class ConnectionManager:
                 self.get_automated_metadata(update_path, root)
                 self.show_information("Removed " + keyword + removed_file + " succesfully", True)
             else:
-                self.log("NETWORK ERROR "+str(resp.status)+" - Removing " + keyword + removed_file + " from " + update_path + " failed.")
-                self.log(">> Reason:", resp.body)
+                self.logger.network_error(str(resp.status) + " - Removing " + keyword + removed_file + " from " + update_path + " failed.")
+                self.logger.network_error(">> Reason:", resp.body)
                 self.show_information("Removing of " + keyword + removed_file + " failed, network error", False)
         else:
             self.hide_loading_ui()
@@ -354,13 +402,13 @@ class ConnectionManager:
                 self.get_automated_metadata(update_path, root)
                 self.show_information("Created folder " + folder_name + " succesfully", True)
             else:
-                self.log("NETWORK ERROR "+str(resp.status)+" - Could not create folder " + full_create_path)
-                self.log(">> Reason:", resp.body)
+                self.logger.network_error(str(resp.status) + " - Could not create folder " + full_create_path)
+                self.logger.network_error(">> Reason:", resp.body)
                 self.show_information("Creating folder " + folder_name + " failed, network error", False)
         else:
             self.hide_loading_ui()
             self.show_information("Creating folder failed, internal error", False)
-        
+
 
 """ NetworkWorker is a simple thread subclass that will do networking.
     Set method with arguments. Set callback function that will be called with self.response
@@ -370,14 +418,19 @@ class NetworkWorker(Thread):
 
     def __init__(self):
         Thread.__init__(self)
-        
         self.method = None
         self.parameters = None
         self.callback = None
         self.callback_parameters = None
         self.response = None
         self.error = None
-
+        
+    def clone(self, worker):
+        self.method = worker.method
+        self.parameters = worker.parameters
+        self.callback = worker.callback
+        self.callback_parameters = worker.callback_parameters
+                        
     def set_callable(self, method, *parameters):
         self.method = method
         self.parameters = parameters
