@@ -1,16 +1,21 @@
 
+import os
+
 from PyQt4.QtGui import QIcon, QPixmap, QImage
+from PyQt4.QtCore import QDir, QString
+
+from ConfigParser import ConfigParser, SafeConfigParser, NoSectionError
 
 """ DataParser gets data from network layer, converts
     to app format and sends onwards to ui layer """
 
 class DataParser:
 
-    def __init__(self, ui_handler):
+    def __init__(self, ui_handler, logger):
         self.ui_handler = ui_handler
         self.tree_controller = ui_handler.tree_controller
         self.controller = ui_handler.controller
-        self.log = self.ui_handler.controller.log
+        self.logger = logger
 
     def parse_metadata(self, data, opened_folders):
         tree_item = None
@@ -23,7 +28,7 @@ class DataParser:
             if data["path"] != "":
                 folder = self.tree_controller.get_folder_for_path(data["path"])
                 if folder == None:
-                    self.log("ERROR - Something went very wrong, did not find folder in parse_metada()!")
+                    self.logger.error("Something went very wrong, did not find folder in parse_metada()!")
                     return tree_item
                 folder.clear_items()
                 folder.hash = data["hash"]
@@ -69,13 +74,13 @@ class DataParser:
                 if data["is_dir"]:
                     if item.tree_item != None:
                         item.tree_item.setExpanded(False)
-                        item.tree_item.setIcon(0, QIcon(self.controller.abs_path("ui/icons/folder_delete.png")))
+                        item.tree_item.setIcon(0, QIcon(self.controller.datahandler.datapath("ui/icons/folder_delete.png")))
                         item.tree_item.setText(1, "deleted")
                     else:
                         item.mime_type = "deleted_folder"
                 else:
                     if item.tree_item != None:
-                        item.tree_item.setIcon(0, QIcon(self.controller.abs_path("ui/icons/cancel.png")))
+                        item.tree_item.setIcon(0, QIcon(self.controller.datahandler.datapath("ui/icons/cancel.png")))
                         item.tree_item.setText(1, "deleted")
                     else:
                         item.mime_type = "deleted_item"
@@ -86,7 +91,7 @@ class DataParser:
     def parse_thumbnail(self, resp, image_path):
         image = QImage.fromData(resp.read())
         if image.isNull():
-            self.log("ERROR - Failed to generate image from raw data for", image_path)
+            self.logger.warning("Failed to generate image from raw data for", image_path)
             self.tree_controller.thumbs[image_path] = None # So we dont come here again
             return
         pixmap = QPixmap.fromImage(image)
@@ -98,15 +103,99 @@ class DataParser:
             account_data = resp.data
             name = account_data["display_name"]
             self.ui_handler.manager_ui.label_username.setText(name)
-            user_icon = QPixmap(self.controller.abs_path("ui/icons/user.png"))
+            user_icon = QPixmap(self.controller.datahandler.datapath("ui/icons/user.png"))
             self.ui_handler.manager_ui.label_username_icon.setPixmap(user_icon.scaled(24,24))
-            self.log("Account data recieved")
+            self.logger.auth("Account data received for " + name)
         else:
             self.ui_handler.manager_ui.label_username.setText("unknown user")
-            user_icon = QPixmap(self.controller.abs_path("ui/icons/user_white.png"))
+            user_icon = QPixmap(self.controller.datahandler.datapath("ui/icons/user_white.png"))
             self.ui_handler.manager_ui.label_username_icon.setPixmap(user_icon.scaled(24,24))
-            self.log("Failed to fetch account data, treating as unknown user")
+            self.logger.auth("Failed to fetch account data, treating as unknown user")
 
+
+""" Maemo data handler """
+
+class MaemoDataHandler:
+
+    def __init__(self, controller, maemo, logger):
+        self.controller = controller
+        self.maemo = maemo
+        self.logger = logger
+        if self.maemo:
+            home = QDir.home().absolutePath()
+            self.app_root = "/opt/dropn900/"
+            self.config_root = str(home) + "/.dropn900/"
+            self.data_root = str(home) + "/MyDocs/DropN900/"
+        else:
+            self.app_root = ""
+            self.config_root = ""
+            self.data_root = ""
+
+    def datapath(self, datafile):
+        return self.app_root + datafile
+        
+    def configpath(self, configfile):
+        return self.config_root + configfile
+        
+    def get_data_dir_path(self):
+        return self.data_root
+        
+    def startup_checks(self):
+        self.logger.info("Maemo startup checks...")
+        if QDir.home().absolutePath() != "/home/user":
+            self.logger.warning(">> home folder != /home/user")
+        self.check_for_data_folder()
+        self.check_for_config_folder()
+        
+    def check_for_data_folder(self):
+        if not self.maemo:
+            return
+        data_dir = QDir.home()
+        if data_dir.cd("MyDocs"):
+            if not data_dir.cd("DropN900"):
+                if data_dir.mkdir("DropN900"):
+                    self.logger.info(">> Created default data dir " + str(data_dir.absolutePath() + "/DropN900"))
+                else:
+                    self.logger.error(">> Could not create default data dir 'DropN900' to " + str(data_dir.absolutePath()))
+            else:
+                self.logger.info(">> Default data dir " + str(data_dir.absolutePath()) + " OK")
+        else:
+            self.logger.error(">> ERROR - Could not find 'MyDocs' folder from " + str(data_dir.absolutePath()))
+
+    def check_for_config_folder(self):
+        if not self.maemo:
+            return
+        config_dir = QDir.home()
+        if not config_dir.cd(".dropn900"):
+            if config_dir.mkdir(".dropn900"):
+                self.logger.info(">> Created config dir '.dropn900' to " + str(config_dir.absolutePath()))
+            else:
+                self.logger.error(">> Could not create config dir '.dropn900' to " + str(config_dir.absolutePath()))
+        else:
+            self.logger.info(">> Config dir " + str(config_dir.absolutePath()) + " OK")
+
+    def store_auth(self, access_token):
+        token_config = ConfigParser()
+        token_config.add_section("token")
+        token_config.set("token", "secret", access_token.secret)
+        token_config.set("token", "key", access_token.key)
+        try:
+            config_file = open(self.configpath("token.ini"), "w")
+            token_config.write(config_file)
+            config_file.close()
+            self.logger.auth("Stored received access token")
+        except IOError:
+            self.logger.error("I/O error while storing received access token, file " + self.configpath("token.ini"))
+        
+    def reset_auth(self):
+        try:
+            os.remove(self.configpath("token.ini"))
+            self.logger.auth("Authentication reseted")
+        except OSError:
+            pass
+        self.controller.start_trusted_auth()
+        
+        
 """ Parent class for all data items """
 
 class Item:
