@@ -1,5 +1,4 @@
 
-import time
 import os
 
 from PyQt4 import QtCore, QtGui
@@ -28,12 +27,16 @@ except ImportError:
 
 class UiController:
 
-    def __init__(self, controller, debug_mode = False):
+    def __init__(self, controller, debug_mode, logger):
         self.controller = controller
+        self.datahandler = controller.datahandler
         self.debug_mode = debug_mode
+        self.logger = logger
 
         ### Main Window
         self.main_widget = QMainWindow()
+        if qt4_maemo5_present == True:
+            self.main_widget.setAttribute(Qt.WA_Maemo5StackedWindow)
         self.main_ui = Ui_DropN900Widget()
         self.main_ui.setupUi(self.main_widget)
 
@@ -73,7 +76,7 @@ class UiController:
         
         network_manager = self.browser_ui.webview.page().networkAccessManager()
         network_manager.sslErrors.connect(self.ssl_errors_occurred)
-        network_manager.finished.connect(self.load_reply_recieved)
+        network_manager.finished.connect(self.load_reply_received)
 
         ### Login Widget
         self.login_widget = QWidget()
@@ -88,7 +91,7 @@ class UiController:
         self.trusted_login_widget = QWidget()
         self.trusted_login_ui = Ui_TrustedLoginWidget()
         self.trusted_login_ui.setupUi(self.trusted_login_widget)
-        self.trusted_login_ui.label_icon.setPixmap(QPixmap(self.controller.abs_path("ui/images/synced_with_dropbox_logo.png")))
+        self.trusted_login_ui.label_icon.setPixmap(QPixmap(self.datahandler.datapath("ui/images/synced_with_dropbox_logo.png")))
 
         # Connects
         self.trusted_login_ui.button_auth.clicked.connect(self.try_trusted_login)
@@ -100,7 +103,7 @@ class UiController:
         
         # Tree Controller
         tree = self.manager_ui.tree_widget
-        self.tree_controller = TreeController(tree, self.manager_ui, self.controller)
+        self.tree_controller = TreeController(tree, self.manager_ui, self.controller, self.logger)
 
         # Hide public link elements (not used yet)
         self.manager_ui.public_link_line_edit.hide()
@@ -115,17 +118,23 @@ class UiController:
         self.manager_ui.button_new_folder.clicked.connect(self.item_new_folder)
 
         ### Console widget
-        self.console_widget = QWidget()
+        
+        if qt4_maemo5_present == True:
+            self.console_widget = QWidget(self.main_widget, Qt.Window)
+            self.console_widget.setAttribute(Qt.WA_Maemo5StackedWindow)
+        else:
+            self.console_widget = QWidget()
         self.console_ui = Ui_ConsoleWidget()
         self.console_ui.setupUi(self.console_widget)
+        self.console_ui.button_save.clicked.connect(self.save_log_to_file)
         self.console_ui.button_back.clicked.connect(self.hide_console)
-        
+
         ### Fill stacked layout
         self.stacked_layout.addWidget(self.browser_widget)
         self.stacked_layout.addWidget(self.login_widget)
         self.stacked_layout.addWidget(self.trusted_login_widget)
         self.stacked_layout.addWidget(self.manager_widget)
-        self.stacked_layout.addWidget(self.console_widget)
+        self.stacked_layout.setCurrentWidget(self.trusted_login_widget)
 
         ### Loading Widget
         self.loading_widget = QWidget(self.manager_widget)
@@ -135,7 +144,7 @@ class UiController:
         self.loading_widget.hide()
 
         # Init loading animation
-        self.loading_animation = QMovie(self.controller.abs_path("ui/images/loading.gif"), "GIF", self.loading_ui.load_animation_label)
+        self.loading_animation = QMovie(self.datahandler.datapath("ui/images/loading.gif"), "GIF", self.loading_ui.load_animation_label)
         self.loading_animation.setCacheMode(QMovie.CacheAll)
         self.loading_animation.setSpeed(150)
         self.loading_animation.setScaledSize(QtCore.QSize(48,48))
@@ -145,10 +154,8 @@ class UiController:
         self.information_message_timer = QTimer()
         self.information_message_timer.setSingleShot(True)
         self.information_message_timer.timeout.connect(self.hide_information_message)
-        self.information_icon_ok = QPixmap(self.controller.abs_path("ui/icons/check.png")).scaled(24,24)
-        self.information_icon_error = QPixmap(self.controller.abs_path("ui/icons/cancel.png")).scaled(24,24)
-
-        self.log("UI initialised...")
+        self.information_icon_ok = QPixmap(self.datahandler.datapath("ui/icons/check.png")).scaled(24,24)
+        self.information_icon_error = QPixmap(self.datahandler.datapath("ui/icons/cancel.png")).scaled(24,24)
 
     def show(self):
         # Nokia N900 screen resolution, full screen
@@ -160,7 +167,7 @@ class UiController:
     def show_about(self):
         if not qt4_maemo5_present:
             return
-        QtMaemo5.QMaemo5InformationBox.information(None, "Made by Jonne Nauha <jonne.nauha@evocativi.com>\nIssue tracker: http://github.com/jonnenauha/DropN900/issues", 0)
+        QtMaemo5.QMaemo5InformationBox.information(None, "Created by Jonne Nauha <jonne.nauha@evocativi.com>\nIssue tracker: http://github.com/jonnenauha/DropN900/issues", 0)
         
     def show_banner(self, message):
         if not qt4_maemo5_present:
@@ -235,12 +242,21 @@ class UiController:
             error = "Password can't be empty!"
             
         if error == None:
-            self.controller.end_trusted_auth(str(email), str(password))
+            self.trusted_login_ui.label_error.setText("Authenticating, please wait...")
+            self.truested_email = str(email)
+            self.trusted_password = str(password)
+            QTimer.singleShot(100, self.do_trusted_login_networking)
         else:
             self.trusted_login_ui.label_error.setText(error)
-                      
+  
+    def do_trusted_login_networking(self):
+        self.controller.end_trusted_auth(self.truested_email, self.trusted_password)
+        self.truested_email = None
+        self.trusted_password = None
+        
     def reset_auth(self):
-        self.controller.reset_auth()
+        self.reset_trusted_login()
+        self.controller.datahandler.reset_auth()
         
     def reset_trusted_login(self):
         self.trusted_login_ui.line_edit_email.clear()
@@ -258,33 +274,12 @@ class UiController:
         if view == "manager":
             widget = self.manager_widget
         if view == "console":
-            widget = self.console_widget
+            self.console_widget.show()
         if view == None and self.last_view != None:
             widget = self.last_view
         if widget != None:
             self.last_view = self.stacked_layout.currentWidget()
             self.stacked_layout.setCurrentWidget(widget)
-    
-    def log(self, msg):
-        timestamp = self.generate_log_timestamp()
-        stamp_mgs = "%s %s" % (timestamp, msg)
-        self.console_ui.text_area.appendPlainText(stamp_mgs)
-        if self.debug_mode:
-            print "%s [DropN900]: %s" % (timestamp, msg)
-            
-    def generate_log_timestamp(self):
-        localtime = time.localtime()
-        hour = localtime.tm_hour
-        if hour < 10:
-            hour = "0" + str(hour)
-        minute = localtime.tm_min
-        if minute < 10:
-            minute = "0" + str(minute)
-        second = localtime.tm_sec
-        if second < 10:
-            second = "0" + str(second)
-        timestamp = "%s:%s:%s" % (hour, minute, second)
-        return timestamp
 
     def get_selected_data(self):
         return self.tree_controller.selected_data
@@ -292,24 +287,33 @@ class UiController:
     # Signal handlers
             
     def shut_down(self):
-        self.log("Shutting down...")
+        self.logger.info("Shutting down...")
         self.main_widget.close()
 
     def show_console(self):
-        if self.action_console.text() == "Show Console":
-            self.action_console.setText("Hide Console")
+        if self.console_widget.isVisible() == False:
             self.switch_context("console")
         else:
             self.hide_console()
-        
+            
+    def save_log_to_file(self):
+        filename = self.datahandler.get_data_dir_path() + "dropn900.log"
+        log_string = str(self.console_ui.text_area.toPlainText())
+        try:
+            log_file = open(filename, "w")
+            log_file.write(log_string)
+            log_file.close()
+            self.show_banner("Log saved to " + filename)
+        except IOError:
+            self.logger.error("Could not open " + filename + " to save log")
+
     def hide_console(self):
-        self.action_console.setText("Show Console")
-        self.stacked_layout.setCurrentWidget(self.last_view)
+        self.console_widget.hide()
 
     # We should not end here in the device as it has ssl libs in place
     # on windows youll have to have openssl libs in place or login via webauth wont work
     def ssl_errors_occurred(self, reply, errors):
-        self.controller.log("NETWORK ERROR - SSL errors while loading", reply.url())
+        self.logger.network_error("SSL errors while loading", str(reply.url()))
         
     def webview_load_url(self, url = None):
         if url == None:
@@ -332,13 +336,13 @@ class UiController:
         self.browser_ui.webview_status_label.setText(status)
         self.browser_ui.url_line_edit.setText(self.browser_ui.webview.url().toString())
 
-    def load_reply_recieved(self, reply):
+    def load_reply_received(self, reply):
         attr_redir = reply.attribute(QNetworkRequest.RedirectionTargetAttribute)
         if not attr_redir.isNull():
             self.browser_ui.webview_status_label.setText("Redirecting...")
         if reply.error() != 0:
-            self.controller.log("NETWORK ERROR -", reply.errorString())
-            self.controller.log(">> Error code", str(reply.error()))
+            self.logger.network_error("NETWORK ERROR -", reply.errorString())
+            self.logger.network_error(">> Error code", str(reply.error()))
 
     def browser_control_clicked(self):
         if self.controller.connected:
@@ -351,7 +355,7 @@ class UiController:
         self.webview_load_url(url)
         self.browser_ui.label_info.hide()
         self.browser_ui.button_done.setText("Back")
-        self.browser_ui.button_done.setIcon(QIcon(self.controller.abs_path("ui/icons/back.png")))
+        self.browser_ui.button_done.setIcon(QIcon(self.datahandler.datapath("ui/icons/back.png")))
         self.switch_context("browser")
 
     def item_download(self):
@@ -361,10 +365,10 @@ class UiController:
 
         # This dialog shows sometimes strange stuff on maemo5
         # It's a PyQt4 bug and its the best we got, you can cope with this
-        local_folder_path = QFileDialog.getExistingDirectory(self.manager_widget, "Select Download Folder", "/home/user/DropN900", QFileDialog.ShowDirsOnly|QFileDialog.HideNameFilterDetails|QFileDialog.ReadOnly)
+        local_folder_path = QFileDialog.getExistingDirectory(self.manager_widget, "Select Download Folder", self.datahandler.get_data_dir_path(), QFileDialog.ShowDirsOnly|QFileDialog.HideNameFilterDetails|QFileDialog.ReadOnly)
         if local_folder_path.isEmpty():
             return
-        
+
         store_path = local_folder_path + "/" + data.name
         self.controller.connection.get_file(data.path, data.root, str(store_path))
         
@@ -464,11 +468,12 @@ class UiController:
 
 class TreeController:
 
-    def __init__(self, tree_widget, controls_ui, controller):
+    def __init__(self, tree_widget, controls_ui, controller, logger):
         self.tree = tree_widget
         self.controls_ui = controls_ui
         self.controller = controller
-        self.log = controller.log
+        self.datahandler = controller.datahandler
+        self.logger = logger
 
         self.root_folder = None
         self.last_clicked = None
@@ -504,7 +509,7 @@ class TreeController:
         self.tree.itemExpanded.connect(self.item_expanded)
         self.tree.itemCollapsed.connect(self.item_collapsed)
 
-    def set_connection(self, connection):
+    def setup(self, connection):
         self.connection = connection
 
     def item_expanded(self, tree_item):
@@ -687,7 +692,7 @@ class TreeController:
             load_widget.setStyleSheet("QLabel { background-color: transparent; }")
 
             # Create animation
-            load_anim = QMovie(self.controller.abs_path("ui/images/loading_tree.gif"), "GIF", load_widget)
+            load_anim = QMovie(self.datahandler.datapath("ui/images/loading_tree.gif"), "GIF", load_widget)
             load_anim.setCacheMode(QMovie.CacheAll)
             load_anim.setSpeed(150)
             load_anim.setScaledSize(QtCore.QSize(30,30))
@@ -703,7 +708,7 @@ class TreeController:
             del child
 
     def set_icon(self, item_type, tree_item, index = 0, label = None):
-        base_path = self.controller.app_root + "ui/icons/"
+        base_path = self.controller.datahandler.app_root + "ui/icons/"
         base_type = item_type.split("/")[0]
         # item type
         if item_type == "folder":
@@ -772,7 +777,7 @@ class TreeController:
             
         # Be sure that something was returned
         if data == None:
-            self.log("ERROR - Could not find data for clicked tree item.")
+            self.logger.error("Could not find data for clicked tree item.")
             return
 
         # Update ui with selected data
@@ -808,7 +813,7 @@ class TreeController:
         # Fetch thumb from web
         else:
             if not self.is_format_supported(image_path):
-                self.log("Format not supported, not fetching thumbnail for", image_path)
+                self.logger.warning("Format not supported, not fetching thumbnail for", image_path)
                 self.thumbs[image_path] = None # So we dont come here again
                 return
             self.connection.get_thumbnail(image_path, root, size)
@@ -839,7 +844,7 @@ class TreeController:
         data = self.selected_data
         # Validity checks
         if data == None:
-            self.log("ERROR - Could not find data for clicked tree item.")
+            self.logger.error("Could not find data for clicked tree item.")
             return
         if not data.is_folder():
             return
